// Enable debug prints to serial monitor
#define MY_DEBUG
// Enable serial gateway
#define MY_GATEWAY_SERIAL
// Define a lower baud rate for Arduino's running on 8 MHz (Arduino Pro Mini 3.3V & SenseBender)
#if F_CPU == 8000000L
#define MY_BAUD_RATE 38400
#endif
// Enable inclusion mode
#define MY_INCLUSION_MODE_FEATURE
// Enable Inclusion mode button on gateway
#define MY_INCLUSION_BUTTON_FEATURE
// Inverses behavior of inclusion button (if using external pullup)
//#define MY_INCLUSION_BUTTON_EXTERNAL_PULLUP
// Set inclusion mode duration (in seconds)
#define MY_INCLUSION_MODE_DURATION 60
// Digital pin used for inclusion mode button
#define MY_INCLUSION_MODE_BUTTON_PIN 3

#include <SPI.h>
#include <MySensors.h>
#include <Bounce2.h>
#include <DallasTemperature.h>
#include <OneWire.h>

// Enable repeater functionality for this node
#define MY_REPEATER_FEATURE

// Dla obsługi przycisków zwykłych
#define D_NUMBER_OF_BUTTONS 23 // max 25 pinów piny 0 i 1 pomijamy, 24 ds18b20, 25 dht22
#define D_FIRST_BUTTON_PIN 4
#define D_FIRST_RELAY_PIN (D_FIRST_BUTTON_PIN + 1)
// 4 -> 5
// 6 -> 7

#define RELAY_ON 1  // GPIO value to write to turn on attached relay
#define RELAY_OFF 0 // GPIO value to write to turn off attached relay

// DS18B20
#define COMPARE_TEMP 0 // Send temperature only if changed? 1 = Yes 0 = No
#define ONE_WIRE_BUS 50 // Pin where dallase sensor is connected 
#define MAX_ATTACHED_DS18B20 7
unsigned long SLEEP_TIME_TEMP = 30000; // Sleep time between reads (in milliseconds)
unsigned long previousMillis = 0; 
OneWire oneWire(ONE_WIRE_BUS); // Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
DallasTemperature sensors(&oneWire); // Pass the oneWire reference to Dallas Temperature. 
byte D[7][8] = {
{ 0x28, 0x10, 0x62, 0x76, 0xE0, 0x01, 0x3C, 0x68 },
{ 0x28, 0x61, 0x81, 0x76, 0xE0, 0x01, 0x3C, 0x53 },
{ 0x28, 0x42, 0xEA, 0x96, 0xF0, 0x01, 0x3C, 0xDE },   // sypialnia
{ 0x28, 0x9E, 0x92, 0x96, 0xF0, 0x01, 0x3C, 0x34 },    // gościnny
{ 0x28, 0x00, 0x09, 0x00, 0x00, 0x95, 0x23, 0x10 },    // skrzynka RPi
{ 0x28, 0x00, 0x09, 0x00, 0x42, 0xD5, 0x23, 0xF5 },
{ 0x28, 0x83, 0x47, 0x48, 0xF6, 0xB2, 0x3C, 0x3B }
};

float lastTemperature[MAX_ATTACHED_DS18B20];
bool receivedConfig = false;
bool metric = true;


// Dla obsługi przycisków zwykłych
Bounce *buttons = new Bounce[D_NUMBER_OF_BUTTONS];

void before()
{
    // Dla zwykłych włączników
    for (int i = 0; i < D_NUMBER_OF_BUTTONS; i++)
    {
        pinMode(D_FIRST_RELAY_PIN + (i * 2), OUTPUT);                                                           // Then set relay pins in output mode
        digitalWrite(D_FIRST_RELAY_PIN + (i * 2), loadState(D_FIRST_RELAY_PIN + (i * 2)) ? RELAY_ON : RELAY_OFF); // Set relay to last known state (using eeprom storage)  
        //digitalWrite(D_FIRST_RELAY_PIN + (i * 2), RELAY_OFF );  
    }
    
    //DS18B20
    sensors.begin();  
}

void setup()
{
    delay(5000);
    
    // Dla włączników zwykłych
    for (int i = 0; i < D_NUMBER_OF_BUTTONS; i++)
    {
        buttons[i].attach(D_FIRST_BUTTON_PIN + (i * 2), INPUT_PULLUP); //setup the bounce instance for the current button
        buttons[i].interval(25);                                     // interval in ms
        pinMode(D_FIRST_BUTTON_PIN + (i * 2), INPUT_PULLUP);
    }

    // DS18B20
    // requestTemperatures() will not block current thread
    sensors.setWaitForConversion(false);
}

void presentation()
{
    // Send the sketch version information to the gateway and Controller
    sendSketchInfo("Arduino - zwykłe", "1.0");

    
    for (int i = 0; i < D_NUMBER_OF_BUTTONS+MAX_ATTACHED_DS18B20; i++)
    {
        // Dla włączników zwykłych
        if (i<=D_NUMBER_OF_BUTTONS){ 
        // Register all sensors to gw (they will be created as child devices)
        present(i, S_LIGHT);
        }
        
        //DS18B20
        else {
          present(i, S_TEMP);          
        }
    }
}

// Dla włączników zwykłych
MyMessage msgLight(1, V_LIGHT);
//DS18B20
MyMessage msgTemp(0,V_TEMP);



void loop()
{
    unsigned long currentMillis = millis();
    
    // Dla włączników zwykłych
    // Send locally attached sensor data here
    for (int i = 0; i < D_NUMBER_OF_BUTTONS; i++)
    {
        // Update the Bounce instance :
        bool zmiana = buttons[i].update();
        if (zmiana)
        {
            int value = buttons[i].read();            
            // Send in the new value.  3,5,7
            uint8_t state = !loadState(D_FIRST_RELAY_PIN + (i * 2));
            digitalWrite(D_FIRST_RELAY_PIN + (i * 2), state ? RELAY_ON : RELAY_OFF);
            saveState(D_FIRST_RELAY_PIN + (i * 2), state);
           
            msgLight.setSensor(i);
            send(msgLight.set(state));  
        } 
    }

    //DS18B20
    if (currentMillis - previousMillis >= SLEEP_TIME_TEMP) 
    {
    // save the last time you blinked the LED
    previousMillis = currentMillis;

    // Fetch temperatures from Dallas sensors
    sensors.requestTemperatures();
  
    // query conversion time and sleep until conversion completed
    int16_t conversionTime = sensors.millisToWaitForConversion(sensors.getResolution());
    // sleep() call can be replaced by wait() call if node need to process incoming messages (or if node is repeater)
    sleep(conversionTime);
  
    // Read temperatures and send them to controller 
    for (int i=0; i<MAX_ATTACHED_DS18B20; i++) {
  
      // Fetch and round temperature to one decimal
      // float temperature = static_cast<float>(static_cast<int>((getConfig().isMetric?sensors.getTempCByIndex(i):sensors.getTempFByIndex(i)) * 10.)) / 10.;
      float temperature = sensors.getTempC(D[i]);
      
      // Only send data if temperature has changed and no error
      if (COMPARE_TEMP == 1) {
        if (lastTemperature[i] != temperature && temperature != -127.00 && temperature != 85.00) {
            // Send in the new temperature
            send(msgTemp.setSensor(i+D_NUMBER_OF_BUTTONS+1).set(temperature,1));
            // Save new temperatures for next compare
            lastTemperature[i]=temperature;
        }
      }
      else {
        if (temperature != -127.00 && temperature != 85.00) {
            // Send in the new temperature
            send(msgTemp.setSensor(i+D_NUMBER_OF_BUTTONS+1).set(temperature,1));
            // Save new temperatures for next compare
            lastTemperature[i]=temperature;
        }
      }
    }
  }    
}

void receive(const MyMessage &message)
{
    // Dla włączników zwykłych
    // We only expect one type of message from controller. But we better check anyway.
    if (message.type == V_LIGHT)
    {
        // Change relay state
        uint8_t state = !loadState(D_FIRST_RELAY_PIN + ((message.sensor) * 2) + 2);
        digitalWrite(D_FIRST_RELAY_PIN + ((message.sensor - 1) * 2) + 2, message.getBool()? RELAY_ON : RELAY_OFF);
        saveState(D_FIRST_RELAY_PIN + ((message.sensor - 1) * 2) + 2, message.getBool()); 
        
        // debug info
        Serial.print("Incoming change for sensor:");
        Serial.print(message.sensor);
        Serial.print(", New status: ");
        Serial.println(message.getBool());
    }
}